@@ -31,6 +31,8 @@
 #include <Lum/Base/Path.h>
 #include <Lum/Base/String.h>
 
+#include <Lum/Def/Menu.h>
+
 #include <Lum/Dlg/About.h>
 #include <Lum/Dlg/ActionDialog.h>
 #include <Lum/Dlg/Progress.h>
@@ -41,13 +43,13 @@
 #include <Lum/Model/String.h>
 #include <Lum/Model/Table.h>
 
+#include <Lum/OS/Color.h>
 #include <Lum/OS/Main.h>
 
 #include <Lum/ButtonRow.h>
 #include <Lum/Dialog.h>
 #include <Lum/Grid.h>
 #include <Lum/Label.h>
-#include <Lum/Menu.h>
 #include <Lum/PercentBar.h>
 #include <Lum/PercentPie.h>
 #include <Lum/Table.h>
@@ -102,13 +104,14 @@ bool GetDisks(std::list<DiskInfo>& list)
     return false;
   }
 
-  while (true) {
-    memset(&stats,0,sizeof(stats));
-    inf=getmntent(file);
-    while ((inf!=NULL) && ((statfs(inf->mnt_dir,&stats)!=0) || (stats.f_blocks==0))) {
-      inf=getmntent(file);
-    }
-    if ((inf!=NULL) && (statfs(inf->mnt_dir,&stats)==0)) {
+  std::cout << "---" << std::endl;
+  memset(&stats,0,sizeof(stats));
+  inf=getmntent(file);
+  while (inf!=NULL) {
+    if (statfs(inf->mnt_dir,&stats)==0 &&
+        stats.f_blocks>0 &&
+        strcmp(inf->mnt_type,"tmpfs")!=0 &&
+        strcmp(inf->mnt_type,"none")!=0) {
       info.name=Lum::Base::StringToWString(inf->mnt_fsname);
       info.dir=Lum::Base::StringToWString(inf->mnt_dir);
       info.blocks=stats.f_blocks;
@@ -116,11 +119,14 @@ bool GetDisks(std::list<DiskInfo>& list)
       info.avail=stats.f_bavail;
       info.unit=stats.f_bsize/1024;
 
+      std::cout << inf->mnt_dir << ": " << inf->mnt_type << std::endl;
+
       list.push_back(info);
     }
     else {
-      break;
+      std::cout << "...skipping " << inf->mnt_dir << std::endl;
     }
+    inf=getmntent(file);
   }
 
   endmntent(file);
@@ -214,7 +220,7 @@ bool GetDisks(std::list<DiskInfo>& list)
       info.dir=buffer2;
 
       if (info.dir==L"OS") {
-	      info.dir=info.name;
+        info.dir=info.name;
       }
     }
 
@@ -241,16 +247,15 @@ std::wstring DoubleToWString(double value, double unit)
 
   buffer.imbue(std::locale(""));
   buffer.setf(std::ios::fixed);
-  buffer<< std::setprecision(1);
+  buffer << std::setprecision(1);
 
-  if (ceil(value*unit)>=1024*1024*1024) {
+  if (ceil(value*unit)>=1024*1024*1024*0.5) {
     buffer << (value*unit)/(1024*1024*1024) << " TB";
   }
-  else if (ceil(value*unit)>=1024*1024) {
+  else if (ceil(value*unit)>=1024*1024*0.5) {
     buffer << (value*unit)/(1024*1024) << " GB";
   }
-  else if (ceil(value*unit)>=1024) {
-    buffer<< std::setprecision(1);
+  else if (ceil(value*unit)>=1024*0.5) {
     buffer << (value*unit)/1024 << " MB";
   }
   else {
@@ -699,26 +704,101 @@ public:
   }
 };
 
+class Usage : public Lum::Control
+{
+private:
+  Lum::Model::DoubleRef model;
+  Lum::OS::FontRef      font;
+  Lum::OS::ColorRef     red;
+  Lum::OS::ColorRef     green;
+
+public:
+  Usage()
+  : font(Lum::OS::display->GetFont()),
+    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+  {
+    // no code
+  }
+
+  bool SetModel(Lum::Base::Model* model)
+  {
+    this->model=dynamic_cast<Lum::Model::Double*>(model);
+
+    Control::SetModel(this->model);
+
+    return this->model.Valid();
+  }
+
+  void CalcSize()
+  {
+    minWidth=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             3*font->StringWidth(L"m",Lum::OS::Font::normal);
+    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+
+    width=minWidth;
+    height=minHeight;
+
+    Control::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Control::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    if (!model.Valid() || model->IsNull()) {
+      return;
+    }
+
+    double percent=model->Get();
+
+    if (percent>=0.9) {
+      draw->PushForeground(red);
+    }
+    else {
+      draw->PushForeground(green);
+    }
+
+    draw->FillRectangle(this->x,this->y+1,
+                        (size_t)(width*percent),height-2);
+
+    draw->PopForeground();
+  }
+
+  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+  {
+    if (model==this->model) {
+      Redraw();
+    }
+    Control::Resync(model,msg);
+  }
+};
+
 typedef Lum::Model::StdTable<DiskInfo>      DiskInfoModel;
 typedef Lum::Base::Reference<DiskInfoModel> DiskInfoModelRef;
 
 class DiskInfoDataProvider : public DiskInfoModel::DataProvider
 {
 private:
-  Lum::PercentBar                 *percent;
-  Lum::Model::DoubleDataStreamRef data;
+  Usage                 *percent;
+  Lum::Model::DoubleRef data;
 
 public:
   DiskInfoDataProvider()
-  : percent(new Lum::PercentBar()),
-    data(new Lum::Model::DoubleDataStream())
+  : percent(new Usage()),
+    data(new Lum::Model::Double())
   {
     percent->SetFlex(true,true);
     percent->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
     percent->SetModel(data);
-
-    data->SetNotificationMode(Lum::Model::DoubleDataStream::notifyExplicit);
-    data->SetChannels(3);
   }
 
   virtual ~DiskInfoDataProvider()
@@ -730,8 +810,6 @@ public:
   {
     switch (column) {
     case 1:
-      return iter->name;
-    case 2:
       return iter->dir;
     default:
       return L"";
@@ -741,11 +819,8 @@ public:
   Lum::Object* GetObject(const DiskInfoModel::Iterator& iter, size_t column) const
   {
     switch (column) {
-    case 3:
-      data->Set(0,(iter->blocks-iter->free)/iter->blocks);
-      data->Set(1,iter->avail/iter->blocks);
-      data->Set(2,(iter->free-iter->avail)/iter->blocks);
-
+    case 2:
+      data->Set((iter->blocks-iter->avail)/iter->blocks);
       return percent;
     default:
       return NULL;
@@ -836,8 +911,7 @@ public:
     panel->AddSpace();
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Dir",Lum::Base::Size::stdCharWidth,10);
-    headerModel->AddColumn(L"Dev",Lum::Base::Size::stdCharWidth,10);
+    headerModel->AddColumn(L"Dir",Lum::Base::Size::stdCharWidth,25);
     headerModel->AddColumn(L"Size",Lum::Base::Size::stdCharWidth,15,true);
 
     table=new Lum::Table();
@@ -852,15 +926,16 @@ public:
 
     SetMain(panel);
 
-    Lum::MenuDesc *menu=new Lum::MenuDesc();
+    Lum::Def::Menu *menu=Lum::Def::Menu::Create();
 
     menu
-      ->AddMenuItemSub(_ld(menuProject))
-        ->AddMenuItemAction(_ld(menuProjectQuit),Lum::OS::qualifierControl,L"q",GetClosedAction());
-    menu
-      ->AddMenuItemSub(_ld(menuHelp))
-        ->AddMenuItemAction(_ld(menuHelpHelp),NULL)
-        ->AddMenuItemAction(_ld(menuHelpAbout),about);
+      ->GroupProject()
+        ->ActionQuit(GetClosedAction())
+      ->End()
+      ->GroupHelp()
+        //->AddMenuItemAction(_ld(menuHelpHelp),NULL)
+        ->ActionAbout(about)
+      ->End();
 
     SetMenu(menu);
 
@@ -944,7 +1019,7 @@ public:
 
       total->Set(DoubleToWString(iter->blocks,iter->unit));
       used->Set(DoubleToWString(iter->blocks-iter->free,iter->unit));
-      free->Set(DoubleToWString(iter->free,iter->unit)+L" ("+DoubleToWString(iter->avail,iter->unit)+L")");
+      free->Set(DoubleToWString(iter->avail,iter->unit)+L" (root: "+DoubleToWString(iter->free,iter->unit)+L")");
 
       data->Enable();
       data->Set(0,(iter->blocks-iter->free)/iter->blocks); // used
@@ -1011,6 +1086,11 @@ public:
   {
     if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
       UpdateDisplay();
+
+      if (tableModel->GetRows()>0) {
+        selection->SelectLine(1);
+      }
+
       Lum::OS::display->AddTimer(refreshTime->Get(),0,timer);
     }
     else if (model==timer && timer->IsFinished()) {
